import { Component, Input, OnInit, OnChanges, ChangeDetectorRef, SimpleChanges } from "@angular/core";
import { AyudaService } from "../../service/ayuda.service";
import { GeneralUtils } from "../../service/general-utils.service";
import { ActivatedRoute } from "@angular/router";
import { SessionSharedService } from "../../service/session-shared.service";

@Component({
  selector: "bit-readonly",
  templateUrl: "bit-readonly.component.html"
})
export class BitReadonlyComponent implements OnInit, OnChanges {
  @Input() ayuda: string;
  @Input() valor: any;
  @Input() tipo: string; // text (default), date, datetime, time, boolean, textarea, list, list-multiple
  @Input() hideLabel: boolean = false;
  @Input() data: any[]; //lista de valores estáticos con lo que inicializamos el select
  @Input() domain: string; //nombre del dominio del que queremos extraer los datos

  private opcionesDropdown: any[] = [];

  constructor(
    protected router: ActivatedRoute,
    protected sessionShared: SessionSharedService,
    protected ayudaService: AyudaService,
    protected changeDetectorRef: ChangeDetectorRef
  ) { }

  ngOnInit() {
    if (!!this.domain && !this.data) {
      this.router.data.subscribe(result => {
        let dominios = result["dominios"];
        let domainTipo = dominios[this.domain];
        if (domainTipo != null) {
          domainTipo.subscribe(
            result => {
              //console.log(result);
              this.sessionShared.set(this.domain, result.slice()); // almacenamos en sesión una copia para evitar transmitir cambios
              this.opcionesDropdown = result.slice();
              this.changeDetectorRef.detectChanges();
            },
            error => {
              console.log('Error carrega dominis', this.domain, error);
            }
          );
        } else {
          console.log("Domini sense dades", this.domain);
        }
      });
    } else if (this.data) {
      this.opcionesDropdown = this.data.slice();
    }
  }

  ngOnChanges(changes: SimpleChanges) {
    for (let propName in changes) {
      if (propName === "data") {
        let newData: any[] = changes[propName].currentValue || [];
        this.opcionesDropdown = newData.slice();
        this.changeDetectorRef.detectChanges();
      }
    }
  }

  showAyuda() {
    this.ayudaService.mostrarAyuda(this.ayuda);
  }

  get value_lectura(): string {
    this.tipo = this.tipo || 'text';
    switch (this.tipo) {
      case 'date':
        return this.valor ? GeneralUtils.getDateAsString(GeneralUtils.stringShortToDate(this.valor)) : '';
      case 'datetime':
        return this.valor ? GeneralUtils.getDateTimeAsString(GeneralUtils.stringToDate(this.valor)) : '';
      case 'time':
        return this.valor ? GeneralUtils.getTimeAsString(GeneralUtils.stringToTime(this.valor)) : '';
      case 'boolean':
        return this.valor ? 'Sí' : 'No';
      case 'list':
        let d = this.opcionesDropdown.find(d => d.value == this.valor);
        return d ? d.label : '';
      case 'list-multiple':
        let labels: any[] = this.opcionesDropdown && this.opcionesDropdown.filter(d => this.valor && this.valor.indexOf(d.value) > -1).map(d => d.label);
        return labels ? labels.join(", ") : null;
      default:
        return this.valor;
    }
  }

  get isTextarea(): boolean {
    return this.tipo === 'textarea';
  }
}